# -*- coding:utf-8 -*-

# import your extensions here, and insert them into __autoload__, minimo
# will auto load the extended cli.

# from .my-extension import my-cli

__autoload__ = (
    # my-cli
)

# end
